
class Metric(object):
    def __init__(self, sku, value):
        self.sku = sku
        self.value = value

    def serialize(self):
        return {
            'sku': self.sku,
            'value': self.value,
        }

class MetricReport(object):
    def __init__(self, metrics, idempotent_key):
        self.metrics = metrics
        self.idempotent_key = idempotent_key

    def serialize(self):
        return {
            'idempotent_key': self.idempotent_key,
            'metrics': [metric.serialize() for metric in self.metrics],
        }


from . import _PROTOCOL_VERSION

class Message(object):
    def __init__(self, data):
        self.version = data.get('version', None)
        self.idempotent_key = data.get('idempotency_key', None)

        self.msg_heart_beat = None
        self.msg_error_report = data.get('error_report', None)
        self.msg_config_validation = data.get('conf_validation', None)
        self.msg_schema_request = data.get('schema_request', None)
        self.msg_request = data.get('request', None)
        self.msg_event = data.get('event', None)

        d = data.get('heartbeat', None)
        if d is not None:
            self.msg_heart_beat = MessageHeartBeat(d)
        d = data.get('error_report', None)
        if d is not None:
            self.msg_error_report = MessageErrorReport(d)
        d = data.get('conf_validation', None)
        if d is not None:
            self.msg_config_validation = MessageConfigValidation(d)
        d = data.get('schema_request', None)
        if d is not None:
            self.msg_schema_request = MessageSchemaRequest(d)
        d = data.get('request', None)
        if d is not None:
            self.msg_request = MessageRequest(d)
        d = data.get('event', None)
        if d is not None:
            self.msg_event = MessageEvent(d)

    def __str__(self):
        return f"Message(version={self.version},idempotent_key={self.idempotent_key},msg_heart_beat={self.msg_heart_beat},msg_error_report={self.msg_error_report},msg_config_validation={self.msg_config_validation},msg_schema_request={self.msg_schema_request},msg_request={self.msg_request},msg_event={self.msg_event})"

class OrgAccessData(object):
    def __init__(self, orgData):
        self.oid = orgData.get( 'oid', None )
        self.jwt = orgData.get( 'jwt', None )

class MessageHeartBeat(object):
    def __init__(self, data):
        pass

class MessageErrorReport(object):
    def __init__(self, data):
        self.error = data.get('error', None)
        self.oid = data.get('oid', None)

class MessageConfigValidation(object):
    def __init__(self, data):
        self.org_access_data = None
        od = data.get('org', None)
        if od:
            self.org_access_data = OrgAccessData(od)
        self.conf = data.get('conf', None)
        

class MessageRequest(object):
    def __init__(self, data):
        self.org_access_data = None
        od = data.get('org', None)
        if od:
            self.org_access_data = OrgAccessData(od)
        self.action = data.get('action', None)
        self.data = data.get('data', None)
        self.conf = data.get('config', None)
        self.resState = data.get('resource_state', None)

class MessageEvent(object):
    def __init__(self, data):
        self.org_access_data = None
        od = data.get('org', None)
        if od:
            self.org_access_data = OrgAccessData(od)
        self.event_name = data.get('event_name', None)
        self.data = data.get('data', None)
        self.conf = data.get('config', None)

class MessageSchemaRequest(object):
    def __init__(self, data):
        pass

class Response(object):
    def __init__(self, error = None, data = None, metrics = None):
        self.error = error
        self.data = data
        self.metrics = metrics
    
    def toJSON(self):
        ret = {
            'version': _PROTOCOL_VERSION,
        }
        if self.error:
            ret['error'] = self.error
        if not self.data:
            ret['data'] = {}
        else:
            ret['data'] = self.data
        if self.metrics:
            ret['metrics'] = self.metrics.serialize()
        return ret



class SchemaObject(object):
    def __init__(self, **kwargs):
        self.Fields = {} # {} of Field name to SchemaElement
        self.Key = None
        self.ListElementName = None
        self.ElementDescription = None
        self.Requirements = [] # [] of [] of Field names

        # legacy
        self.RenderType = None
        self.KeyDataType = None # SchemaDataTypes
        self.KeyLabel = None # SchemaDataTypes
        self.KeyDisplayIndex = None # SchemaDataTypes
        self.KeyName = None
        for k, v in kwargs.items():
            if not hasattr(self, k):
                raise Exception(f"unknown attribute {k}")
            setattr(self, k, v)

    def serialize(self):
        return {
            'fields' : {n: f.serialize() for n, f in self.Fields.items()},
            'list_element_name': self.ListElementName,
            'element_desc': self.ElementDescription,
            'key': self.Key,
            'requirements' : self.Requirements,

            # legacy
            'render_type' : self.RenderType,
            'key_data_type' : self.KeyDataType,
            'key_name' : self.KeyName,
            'key_label' : self.KeyLabel,
            'key_display_index' : self.KeyDisplayIndex,
        }

class SchemaElement(object):
    def __init__(self, **kwargs):
        self.Label = ""
        self.Description = ""
        self.DataType = None # SchemaDataTypes
        self.IsList = False
        self.DisplayIndex = None
        self.DefaultValue = None
        self.ObjectSchema = None # SchemaObject
        self.EnumValues = None # [] of string
        self.PlaceHolder = None
        self.SupportedActions = None # [] of Action names
        for k, v in kwargs.items():
            if not hasattr(self, k):
                raise Exception(f"unknown attribute {k}")
            setattr(self, k, v)

    def serialize(self):
        return {
            'label' : self.Label,
            'description' : self.Description,
            'data_type' : self.DataType,
            'is_list' : self.IsList,
            'display_index' : self.DisplayIndex,
            'default_value' : self.DefaultValue,
            'object' : None if self.ObjectSchema is None else self.ObjectSchema.serialize(),
            'enum_values' : self.EnumValues,
            'placeholder' : self.PlaceHolder,
            'supported_actions' : self.SupportedActions,
        }

class RequestSchemas(object):
    def __init__(self):
        self.Actions = {} # {} of Action name to RequestSchema

    def serialize(self):
        return {n : a.serialize() for n, a in self.Actions.items()}

class RequestSchema(object):
    def __init__(self, **kwargs):
        self.IsDefaultRequest = False
        self.IsUserFacing = True
        self.ShortDescription = ""
        self.LongDescription = ""
        self.IsImpersonated = False
        self.ParameterDefinitions = SchemaObject()
        self.ResponseDefinition = None # SchemaObject
        for k, v in kwargs.items():
            if not hasattr(self, k):
                raise Exception(f"unknown attribute {k}")
            setattr(self, k, v)

    def serialize(self):
        return {
            'is_default' : self.IsDefaultRequest,
            'is_user_facing' : self.IsUserFacing,
            'short_description' : self.ShortDescription,
            'long_description' : self.LongDescription,
            'is_impersonated' : self.IsImpersonated,
            'parameters' : self.ParameterDefinitions.serialize(),
            'response' : None if self.ResponseDefinition is None else self.ResponseDefinition.serialize(),
        }

class SchemaView(object):
    def __init__(self, **kwargs):
        self.Name = ""
        self.LayoutType = None
        self.DefaultRequests = None # [] of Request names

    def serialize(self):
        return {
            'name' :self.Name,
            'layout_type' : self.LayoutType,
            'default_requests' : self.DefaultRequests,
        }

class SchemaDataTypes(object):
    String = "string"
    Integer = "integer"
    Boolean = "bool"
    Enum = "enum"
    SensorID = "sid"
    OrgID = "oid"
    Platform = "platform"
    Architecture = "architecture"
    SensorSelector = "sensor_selector"
    EventName = 'event_name'
    Tag = "tag"
    Duration = "duration" # milliseconds
    Time = "time" # milliseconds since epoch
    URL = "url"
    Domain = "domain"
    JSON = "json"
    YAML = "yaml"
    YaraRule = "yara_rule"
    YaraRuleName = "yara_rule_name"
    Object = "object"
    Record = "record"